from PyQt5.Qt import QColor
import argparse
import sys
import os
import numpy
from LegoBrick import LegoBrick
from asyncio.tasks import sleep
from inputFile import InputFile

#A debug function - prints the tower array in a sensible way
def dbgprint(array):
	for h in array:
		for x in h:
			for y in x:
				if(y == 0):
					sys.stdout.write("0")
				else:
					sys.stdout.write("X")
			print()
		print("---")

def process_files(path: str):
	files_to_process = []
	if(os.path.isdir(path)):
		for file in os.listdir(path=path):
			files_to_process.append(file)
	else:
		files_to_process.append(path)
	
	files = []
	
	for infile_path in files_to_process:
		lines = []
		
		infile = open(infile_path, mode='r')
		for line in infile:
			lines.append(line)
			
		towerX = int(lines[0].split()[0])
		towerY = int(lines[0].split()[1])
		towermode = lines[0].split()[2]
		
		legopieces = []
		for i in range(1, lines.__len__() - 1):
			piece_colourcode = lines[i].split()[0]
			if piece_colourcode == "A":				
				piece_colour = QColor.fromRgb(0, 255, 255)
			elif piece_colourcode == "R":
				piece_colour = QColor.fromRgb(255, 0, 0)
			elif piece_colourcode == "G":
				piece_colour = QColor.fromRgb(0, 255, 0)
			elif piece_colourcode == "B":
				piece_colour = QColor.fromRgb(0, 0, 255)
			elif piece_colourcode == "Y":
				piece_colour = QColor.fromRgb(255, 255, 0)
			
			pieceSize = lines[i].split()[1]
			pieceX = int(pieceSize.split("X")[0])
			pieceY = int(pieceSize.split("X")[1])
			pieceCount = int(lines[i].split()[2])
			
			piece = LegoBrick(pieceX, pieceY, piece_colour, pieceCount)
			legopieces.append(piece)
		
		#Sort from biggest to smallest
		legopieces.sort(key=lambda piece: piece.x * piece.y, reverse=True)
		files.append(InputFile(towerX, towerY, towermode, legopieces, infile_path))
	return files
	

#Parse arguments
aparser = argparse.ArgumentParser()
aparser.add_argument("--plan", help="Generate a plan")
args = aparser.parse_args()

#If I should make a plan
if(args.plan != None):
	files = process_files(args.plan)
	
	for file in files:
		towermode = file.towermode
		towerX = file.towerX
		towerY = file.towerY
		legopieces = file.legopieces
		
		planfile = open(file.filename + ".10", mode='w')
		planfile.write(str(towerX) + " " + str(towerY) + " " + towermode + "\n")
		
		#	
		tower = numpy.zeros((20, towerX, towerY), dtype=object)
		
		if(towermode == "P"):
			#Height
			workH = 0;
			workX = 0;
			workY = 0;
			while True:
				if(tower[workH][workX][workY] == 0):
					#I don't want to try all the pieces
					try:
						for piece in legopieces:
							def test_piece(piece_x, piece_y):
								#Can I place this piece here?
								can_place = True
								for x in range(0, piece_x):
									for y in range(0, piece_y):
										#Make sure that that isn't outside the array
										x_fine = x + workX < tower.shape[1]
										y_fine = y + workY < tower.shape[2]
										if(not y_fine or not x_fine or tower[workH][x + workX][y + workY] != 0):
											can_place = False;
								#Place the piece down
								if(can_place == True):
									planfile.write(str(workH + 1) + " " + str(workX + 1) + " " + str(workY + 1) + " " + piece.colour_code() + " " + str(piece.x) + "X" + str(piece.y) + "\n")
									for x in range(0, piece_x):
										for y in range(0, piece_y):
											#The piece itself is assigned to that place because of colours
											tower[workH][x + workX][y + workY] = piece
							
									#Remove the piece
									if(piece.count >= 1):
										piece.count -= 1
									#Remove the piece object entirely if I used all of the pieces
									else:
										legopieces.remove(piece)
							
									legopieces.sort(key=lambda piece: piece.x * piece.y, reverse=True)

									raise StopIteration
						
							test_piece(piece.x, piece.y)
							#Also test the rotated piece
							test_piece(piece.y, piece.x)
					
					except StopIteration:
							pass
					
				if(workX + 1 >= tower.shape[1]):
					workX = 0
					workY += 1
				else:
					workX += 1
				if(workY + 1 > tower.shape[2]):
					workY = 0
					workH += 1
				if(workH >= tower.shape[0]):
					break
			
					
